@@ -71,7 +71,7 @@ async fn main() {
 	info!("Node public address: {:?}", public_addr);
 	info!("Node listen address: {}", listen_addr);
 
-	let netapp = NetApp::new(netid.clone(), privkey.clone());
+	let netapp = NetApp::new(0u64, netid.clone(), privkey.clone());
 
 	let mut bootstrap_peers = vec![];
 	for peer in opt.bootstrap_peers.iter() {
