@@ -9,11 +9,13 @@ use async_trait::async_trait;
 use log::{debug, info, trace, warn};
 use serde::{Deserialize, Serialize};
 
+use tokio::select;
 use tokio::sync::watch;
 
 use sodiumoxide::crypto::hash;
 
 use crate::endpoint::*;
+use crate::error::*;
 use crate::netapp::*;
 use crate::proto::*;
 use crate::NodeID;
@@ -22,6 +24,8 @@ const CONN_RETRY_INTERVAL: Duration = Duration::from_secs(30);
 const CONN_MAX_RETRIES: usize = 10;
 const PING_INTERVAL: Duration = Duration::from_secs(10);
 const LOOP_DELAY: Duration = Duration::from_secs(1);
+const PING_TIMEOUT: Duration = Duration::from_secs(5);
+const FAILED_PING_THRESHOLD: usize = 3;
 
 // -- Protocol messages --
 
@@ -48,10 +52,17 @@ impl Message for PeerListMessage {
 
 #[derive(Debug)]
 struct PeerInfoInternal {
+	// addr is the currently connected address,
+	// or the last address we were connected to,
+	// or an arbitrary address some other peer gave us
 	addr: SocketAddr,
+	// all_addrs contains all of the addresses everyone gave us
+	all_addrs: Vec<SocketAddr>,
+
 	state: PeerConnState,
 	last_seen: Option<Instant>,
 	ping: VecDeque<Duration>,
+	failed_pings: usize,
 }
 
 #[derive(Copy, Clone, Debug)]
@@ -175,9 +186,11 @@ impl FullMeshPeeringStrategy {
 					id,
 					PeerInfoInternal {
 						addr,
+						all_addrs: vec![addr],
 						state: PeerConnState::Waiting(0, Instant::now()),
 						last_seen: None,
 						ping: VecDeque::new(),
+						failed_pings: 0,
 					},
 				);
 			}
@@ -188,9 +201,11 @@ impl FullMeshPeeringStrategy {
 				netapp.id,
 				PeerInfoInternal {
 					addr,
+					all_addrs: vec![addr],
 					state: PeerConnState::Ourself,
 					last_seen: None,
 					ping: VecDeque::new(),
+					failed_pings: 0,
 				},
 			);
 		}
@@ -236,7 +251,7 @@ impl FullMeshPeeringStrategy {
 				let mut to_ping = vec![];
 				let mut to_retry = vec![];
 				for (id, info) in known_hosts.list.iter() {
-					trace!("{}, {:?}", hex::encode(id), info);
+					trace!("{}, {:?}", hex::encode(&id[..8]), info);
 					match info.state {
 						PeerConnState::Connected => {
 							let must_ping = match info.last_seen {
@@ -273,12 +288,23 @@ impl FullMeshPeeringStrategy {
 						if let PeerConnState::Waiting(i, _) = h.state {
 							info!(
 								"Retrying connection to {} at {} ({})",
-								hex::encode(&id),
-								h.addr,
+								hex::encode(&id[..8]),
+								h.all_addrs
+									.iter()
+									.map(|x| format!("{}", x))
+									.collect::<Vec<_>>()
+									.join(", "),
 								i + 1
 							);
 							h.state = PeerConnState::Trying(i);
-							tokio::spawn(self.clone().try_connect(id, h.addr));
+
+							let alternate_addrs = h
+								.all_addrs
+								.iter()
+								.filter(|x| **x != h.addr)
+								.cloned()
+								.collect::<Vec<_>>();
+							tokio::spawn(self.clone().try_connect(id, h.addr, alternate_addrs));
 						}
 					}
 				}
@@ -344,21 +370,42 @@ impl FullMeshPeeringStrategy {
 		debug!(
 			"Sending ping {} to {} at {:?}",
 			ping_id,
-			hex::encode(id),
+			hex::encode(&id[..8]),
 			ping_time
 		);
-		match self.ping_endpoint.call(&id, &ping_msg, PRIO_HIGH).await {
-			Err(e) => warn!("Error pinging {}: {}", hex::encode(id), e),
+		let ping_response = select! {
+			r = self.ping_endpoint.call(&id, &ping_msg, PRIO_HIGH) => r,
+			_ = tokio::time::sleep(PING_TIMEOUT) => Err(Error::Message("Ping timeout".into())),
+		};
+
+		match ping_response {
+			Err(e) => {
+				warn!("Error pinging {}: {}", hex::encode(&id[..8]), e);
+				let mut known_hosts = self.known_hosts.write().unwrap();
+				if let Some(host) = known_hosts.list.get_mut(&id) {
+					host.failed_pings += 1;
+					if host.failed_pings > FAILED_PING_THRESHOLD {
+						warn!(
+							"Too many failed pings from {}, closing connection.",
+							hex::encode(&id[..8])
+						);
+						// this will later update info in known_hosts
+						// through the disconnection handler
+						self.netapp.disconnect(&id);
+					}
+				}
+			}
 			Ok(ping_resp) => {
 				let resp_time = Instant::now();
 				debug!(
 					"Got ping response from {} at {:?}",
-					hex::encode(id),
+					hex::encode(&id[..8]),
 					resp_time
 				);
 				{
 					let mut known_hosts = self.known_hosts.write().unwrap();
 					if let Some(host) = known_hosts.list.get_mut(&id) {
+						host.failed_pings = 0;
 						host.last_seen = Some(resp_time);
 						host.ping.push_back(resp_time - ping_time);
 						while host.ping.len() > 10 {
@@ -394,7 +441,12 @@ impl FullMeshPeeringStrategy {
 
 		let mut changed = false;
 		for (id, addr) in list.iter() {
-			if !known_hosts.list.contains_key(id) {
+			if let Some(kh) = known_hosts.list.get_mut(id) {
+				if !kh.all_addrs.contains(addr) {
+					kh.all_addrs.push(*addr);
+					changed = true;
+				}
+			} else {
 				known_hosts.list.insert(*id, self.new_peer(id, *addr));
 				changed = true;
 			}
@@ -406,10 +458,42 @@ impl FullMeshPeeringStrategy {
 		}
 	}
 
-	async fn try_connect(self: Arc<Self>, id: NodeID, addr: SocketAddr) {
-		let conn_result = self.netapp.clone().try_connect(addr, id).await;
-		if let Err(e) = conn_result {
-			warn!("Error connecting to {}: {}", hex::encode(id), e);
+	async fn try_connect(
+		self: Arc<Self>,
+		id: NodeID,
+		default_addr: SocketAddr,
+		alternate_addrs: Vec<SocketAddr>,
+	) {
+		let conn_addr = {
+			let mut ret = None;
+			for addr in [default_addr].iter().chain(alternate_addrs.iter()) {
+				debug!("Trying address {} for peer {}", addr, hex::encode(&id[..8]));
+				match self.netapp.clone().try_connect(*addr, id).await {
+					Ok(()) => {
+						ret = Some(*addr);
+						break;
+					}
+					Err(e) => {
+						debug!(
+							"Error connecting to {} at {}: {}",
+							hex::encode(&id[..8]),
+							addr,
+							e
+						);
+					}
+				}
+			}
+			ret
+		};
+
+		if let Some(ok_addr) = conn_addr {
+			self.on_connected(id, ok_addr, false);
+		} else {
+			warn!(
+				"Could not connect to peer {} ({} addresses tried)",
+				hex::encode(&id[..8]),
+				1 + alternate_addrs.len()
+			);
 			let mut known_hosts = self.known_hosts.write().unwrap();
 			if let Some(host) = known_hosts.list.get_mut(&id) {
 				host.state = match host.state {
@@ -424,8 +508,6 @@ impl FullMeshPeeringStrategy {
 				};
 				self.update_public_peer_list(&known_hosts);
 			}
-		} else {
-			self.on_connected(id, addr, false);
 		}
 	}
 
@@ -438,19 +520,28 @@ impl FullMeshPeeringStrategy {
 				self.update_public_peer_list(&known_hosts);
 			}
 		} else {
-			info!("Successfully connected to {} at {}", hex::encode(&id), addr);
+			info!(
+				"Successfully connected to {} at {}",
+				hex::encode(&id[..8]),
+				addr
+			);
 			let mut known_hosts = self.known_hosts.write().unwrap();
 			if let Some(host) = known_hosts.list.get_mut(&id) {
 				host.state = PeerConnState::Connected;
 				host.addr = addr;
+				if !host.all_addrs.contains(&addr) {
+					host.all_addrs.push(addr);
+				}
 			} else {
 				known_hosts.list.insert(
 					id,
 					PeerInfoInternal {
 						state: PeerConnState::Connected,
 						addr,
+						all_addrs: vec![addr],
 						last_seen: None,
 						ping: VecDeque::new(),
+						failed_pings: 0,
 					},
 				);
 			}
@@ -461,7 +552,7 @@ impl FullMeshPeeringStrategy {
 
 	fn on_disconnected(self: Arc<Self>, id: NodeID, is_incoming: bool) {
 		if !is_incoming {
-			info!("Connection to {} was closed", hex::encode(id));
+			info!("Connection to {} was closed", hex::encode(&id[..8]));
 			let mut known_hosts = self.known_hosts.write().unwrap();
 			if let Some(host) = known_hosts.list.get_mut(&id) {
 				host.state = PeerConnState::Waiting(0, Instant::now());
@@ -479,9 +570,11 @@ impl FullMeshPeeringStrategy {
 		};
 		PeerInfoInternal {
 			addr,
+			all_addrs: vec![addr],
 			state,
 			last_seen: None,
 			ping: VecDeque::new(),
+			failed_pings: 0,
 		}
 	}
 }
@@ -493,7 +586,7 @@ impl EndpointHandler<PingMessage> for FullMeshPeeringStrategy {
 			id: ping.id,
 			peer_list_hash: self.known_hosts.read().unwrap().hash,
 		};
-		debug!("Ping from {}", hex::encode(&from));
+		debug!("Ping from {}", hex::encode(&from[..8]));
 		ping_resp
 	}
 }
